/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 * 
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tud.kom.p2psim.impl.topology.movement.modularosm.attraction;

import java.util.LinkedList;
import java.util.List;
import de.tudarmstadt.maki.simonstrator.api.Monitor;
import de.tudarmstadt.maki.simonstrator.api.Monitor.Level;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.location.AttractionPoint;

/**
 * Interface for {@link AttractionPoint} generators. Will generate a given amount of attraction points.
 * 
 * @author Christoph Muenker, Julian Zobel
 * @version 1.1, 09 2018
 */
public interface IAttractionGenerator {
	
	// TODO make this private in JAVA >= 9 
	static LinkedList<AttractionPoint> attractionPoints = new LinkedList<>();
	
	public List<AttractionPoint> getAttractionPoints();
	
	/**
	 * Remove an {@link AttractionPoint} from the list
	 * @param ap
	 */
	default void removeAttractionPoint(AttractionPoint ap) {
		
		if(Monitor.hasAnalyzer(AttractionPointMonitor.class)) {
			Monitor.getOrNull(AttractionPointMonitor.class).removedAttractionPoint(ap);
		}
		
		attractionPoints.remove(ap);
	}
	
	/**
	 * Add an {@link AttractionPoint} to the list
	 * @param ap
	 */
	default void addAttractionPoint(AttractionPoint ap) {
		
		if(ap == null) {
			Monitor.log(IAttractionGenerator.class, Level.ERROR, "IAttractionGenerator: Tried to add NULL as Attraction Point");
			return; 
		}
			
		
		if(Monitor.hasAnalyzer(AttractionPointMonitor.class)) {
			Monitor.getOrNull(AttractionPointMonitor.class).addedAttractionPoint(ap);
		}
		
		attractionPoints.add(ap);
	}
}
