/*
 * Copyright (c) 2005-2010 KOM – Multimedia Communications Lab
 *
 * This file is part of PeerfactSim.KOM.
 * 
 * PeerfactSim.KOM is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * any later version.
 * 
 * PeerfactSim.KOM is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with PeerfactSim.KOM.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

package de.tud.kom.p2psim.impl.topology.movement.modularosm.transition;

import java.util.LinkedList;
import java.util.List;
import de.tud.kom.p2psim.api.topology.movement.SimLocationActuator;
import de.tud.kom.p2psim.impl.topology.movement.modularosm.attraction.IAttractionGenerator;
import de.tudarmstadt.maki.simonstrator.api.Event;
import de.tudarmstadt.maki.simonstrator.api.EventHandler;
import de.tudarmstadt.maki.simonstrator.api.component.sensor.location.AttractionPoint;

/**
 * A simplified transition strategy taking only the weights of an AP into account.
 * 
 * @author Bjoern Richerzhagen
 * @version 1.0, Jan 16, 2017
 * 
 * Made the transition strategy extend the {@link AbstractAttractionBasedTransitionStrategy} for a more generalized approach on 
 * the {@link ITransitionStrategy} implementation for attraction point based strategies.
 * 
 * @author Julian Zobel
 * @version 1.1, January 2019
 * 
 */
public class WeightedTransitionStrategy extends AbstractAttractionBasedTransitionStrategy implements EventHandler {
	
	@Override
	public void addComponent(SimLocationActuator ms) {
		this.assignments.put(ms, getNewAssignment(ms));
	}
	
	private AttractionPoint getNewAssignment(SimLocationActuator component) {
		double score = rnd.nextDouble();
		List<AttractionPoint> candidates = new LinkedList<>();
		for (AttractionPoint ap : IAttractionGenerator.attractionPoints) {
			if (ap.getWeight() >= score) {
				candidates.add(ap);
			}
		}
		if (candidates.isEmpty()) {
			candidates.addAll(IAttractionGenerator.attractionPoints);
		}
		AttractionPoint assignment = candidates.get(rnd.nextInt(candidates.size()));
		notifyListenersOfAssignmentUpdate(component,  assignment);
		return assignment;
	}
		
	@Override
	public void eventOccurred(Object content, int type) {
		assert type == EVENT_PAUSE_ENDED;
		SimLocationActuator comp = (SimLocationActuator) content;
		if (!this.assignments.containsKey(comp)) {
			// maybe someone assigned a new AP in the meantime. ignore.
			this.assignments.put(comp, getNewAssignment(comp));
		}
	}

	@Override
	public void reachedAttractionPoint(SimLocationActuator ms) {	
		// JZ: Added this to prevent multiple calls while pausing (otherwise most nodes will not wait more the minimum pause time)
		if (!this.assignments.containsKey(ms)) {			
			return;
		}		
		
		this.lastAssignments.put(ms, this.assignments.remove(ms));
		Event.scheduleWithDelay(getPauseTime(), this, ms, EVENT_PAUSE_ENDED);
	}

}
